@@ -1,6 +1,5 @@
 /*
- *	ttyrpld/k_linux-2.6/rpldev.c
- *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2004 - 2008
+ *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2004 - 2009
  *
  *	This file is part of ttyrpld. ttyrpld is free software; you can
  *	redistribute it and/or modify it under the terms of the GNU
@@ -90,7 +89,7 @@ static inline uint32_t mkdev_26(unsigned long, unsigned long);
 
 /* Variables */
 static DECLARE_WAIT_QUEUE_HEAD(Pull_queue);
-static DECLARE_MUTEX(Buffer_lock);
+static struct semaphore Buffer_lock;
 static spinlock_t Open_lock;
 static char *Buffer = NULL, *BufRP = NULL, *BufWP = NULL;
 static unsigned int Bufsize = 32 * 1024;
@@ -127,6 +126,7 @@ static __init int rpldev_init(void)
 {
 	int ret;
 
+	sema_init(&Buffer_lock, 1);
 	spin_lock_init(&Open_lock);
 	kmi_miscinfo.minor = Minor_nr;
 	if ((ret = misc_register(&kmi_miscinfo)) != 0)
@@ -140,7 +140,6 @@ static __init int rpldev_init(void)
 static __exit void rpldev_exit(void)
 {
 	misc_deregister(&kmi_miscinfo);
-	return;
 }
 
 module_init(rpldev_init);
@@ -165,9 +164,8 @@ static int rpldhc_open(const struct tty_struct *tty, const struct file *filp)
 
 	SKIP_PTM(tty);
 
+	p.evmagic.n = cpu_to_be32(RPLEVT_OPEN);
 	p.dev   = TTY_DEVNR(tty);
-	p.event = EVT_OPEN;
-	p.magic = MAGIC_SIG;
 	fill_time(&p.time);
 
 	/*
@@ -180,7 +178,8 @@ static int rpldhc_open(const struct tty_struct *tty, const struct file *filp)
 		return circular_put_packet(&p, NULL, 0);
 	}
 
-	p.size = cpu_to_le16(len = strlen(full_dev));
+	len    = strlen(full_dev);
+	p.size = cpu_to_le32(len);
 	return circular_put_packet(&p, full_dev, len);
 }
 
@@ -199,10 +198,9 @@ static int rpldhc_read(const char __user *buf, size_t count,
 	if (count == 0)
 		return 0;
 
+	p.evmagic.n = cpu_to_be32(RPLEVT_READ);
 	p.dev   = TTY_DEVNR(tty);
-	p.size  = cpu_to_le16(count);
-	p.event = EVT_READ;
-	p.magic = MAGIC_SIG;
+	p.size  = cpu_to_le32(count);
 	fill_time(&p.time);
 	return circular_put_packet(&p, buf, count);
 }
@@ -215,12 +213,12 @@ static int rpldhc_write(const char __user *buf, size_t count,
 	 * There are two ways an application can use a tty:
 	 *
 	 * 1. noecho mode (the case with interactive shells) Shells print chars
-	 *    as they see fit, thus generating EVT_READ for you hitting a key,
-	 *    and EVT_WRITE for the shell displaying it.
+	 *    as they see fit, thus generating RPLEVT_READ for you hitting a
+	 *    key, and RPLEVT_WRITE for the shell displaying it.
 	 *
 	 * 2. echo mode (cat waiting for EOF on stdin) The _tty driver_ echoes
-	 *    _single_ chars back. This generates EVT_READ for your keyboard
-	 *    interaction, plus _one_ EVT_WRITE when an end-of-line is
+	 *    _single_ chars back. This generates RPLEVT_READ for your keyboard
+	 *    interaction, plus _one_ RPLEVT_WRITE when an end-of-line is
 	 *    received.
 	 */
 	struct rpldev_packet p;
@@ -229,10 +227,9 @@ static int rpldhc_write(const char __user *buf, size_t count,
 	if (count == 0)
 		return 0;
 
+	p.evmagic.n = cpu_to_be32(RPLEVT_WRITE);
 	p.dev   = TTY_DEVNR(tty);
-	p.size  = cpu_to_le16(count);
-	p.event = EVT_WRITE;
-	p.magic = MAGIC_SIG;
+	p.size  = cpu_to_le32(count);
 	fill_time(&p.time);
 	return circular_put_packet(&p, buf, count);
 }
@@ -245,10 +242,9 @@ static int rpldhc_lclose(const struct tty_struct *tty,
 	if (IS_PTY_MASTER(tty))
 		tty = other;
 
+	p.evmagic.n = cpu_to_be32(RPLEVT_LCLOSE);
 	p.dev   = TTY_DEVNR(tty);
 	p.size  = 0;
-	p.event = EVT_LCLOSE;
-	p.magic = MAGIC_SIG;
 	fill_time(&p.time);
 	return circular_put_packet(&p, NULL, 0);
 }
@@ -502,8 +498,6 @@ static inline void circular_put(const void __kernel *src, size_t count)
 		memcpy(Buffer, src + x, count - x);
 		BufWP = Buffer + count - x;
 	}
-
-	return;
 }
 
 static inline void circular_putU(const void __user *src, size_t count)
@@ -518,8 +512,6 @@ static inline void circular_putU(const void __user *src, size_t count)
 		    copy_from_user(Buffer, src + x, count - x) == 0)
 			BufWP = Buffer + count - x;
 	}
-
-	return;
 }
 
 /*
@@ -575,8 +567,6 @@ static inline void fill_time(struct rpltime *x)
 		x->tv_usec = cpu_to_le32(tv.tv_usec);
 	else if (sizeof(tv.tv_usec) == sizeof(uint64_t))
 		x->tv_usec = cpu_to_le64(tv.tv_usec);
-
-	return;
 }
 
 static inline unsigned int min_uint(unsigned int a, unsigned int b)
