@@ -1,7 +1,5 @@
 /*
- *	ttyrpld/k_solaris-11/rpldev.c
- *	Copyright © CC Computer Consultants GmbH, 2006 - 2007
- *	Contact: Jan Engelhardt <jengelh [at] computergmbh de>
+ *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2006 - 2009
  *
  *	The contents of this file are subject to the terms of the Common
  *	Development and Distribution License, Version 1.0 only (the "License").
@@ -54,6 +52,7 @@ static int rpldhc_lclose(const struct queue *);
 static int rpldev_open(dev_t *, int, int, struct cred *);
 static int rpldev_close(dev_t, int, int, struct cred *);
 static int rpldev_read(dev_t, struct uio *, struct cred *);
+static int rpldev_ioctl(dev_t, int, intptr_t, int, struct cred *, int *);
 static int rpldev_chpoll(dev_t, short, int, short *, struct pollhead **);
 
 /* Functions */
@@ -63,7 +62,7 @@ static inline size_t avail_W(void);
 static inline int circular_get(struct uio *, size_t);
 static inline void circular_put(const void *, size_t);
 static int circular_put_packet(struct rpldev_packet *, const void *, size_t);
-static inline void fill_time(struct timeval *);
+static inline void fill_time(struct rpltime *);
 static inline unsigned int min_uint(unsigned int, unsigned int);
 static inline uint32_t mkdev_26(unsigned long, unsigned long);
 
@@ -202,10 +201,9 @@ static int rpldhc_open(const struct queue *q)
 {
 	struct rpldev_packet p;
 
+	p.evmagic.n = BE_32(RPLEVT_OPEN);
 	p.dev   = TTY_DEVNR(q);
 	p.size  = 0;
-	p.event = EVT_OPEN;
-	p.magic = MAGIC_SIG;
 	fill_time(&p.time);
 	return circular_put_packet(&p, NULL, 0);
 }
@@ -217,10 +215,9 @@ static int rpldhc_read(const char *buf, size_t count, const struct queue *q)
 	if (count == 0)
 		return 0;
 
+	p.evmagic.n = BE_32(RPLEVT_READ);
 	p.dev   = TTY_DEVNR(q);
-	p.size  = LE_16(count);
-	p.event = EVT_READ;
-	p.magic = MAGIC_SIG;
+	p.size  = LE_32(count);
 	fill_time(&p.time);
 	return circular_put_packet(&p, buf, count);
 }
@@ -232,10 +229,9 @@ static int rpldhc_write(const char *buf, size_t count, const struct queue *q)
 	if (count == 0)
 		return 0;
 
+	p.evmagic.n = BE_32(RPLEVT_WRITE);
 	p.dev   = TTY_DEVNR(q);
-	p.size  = LE_16(count);
-	p.event = EVT_WRITE;
-	p.magic = MAGIC_SIG;
+	p.size  = LE_32(count);
 	fill_time(&p.time);
 	return circular_put_packet(&p, buf, count);
 }
@@ -244,10 +240,9 @@ static int rpldhc_lclose(const struct queue *q)
 {
 	struct rpldev_packet p;
 
+	p.evmagic.n = BE_32(RPLEVT_LCLOSE);
 	p.dev   = TTY_DEVNR(q);
 	p.size  = 0;
-	p.event = EVT_LCLOSE;
-	p.magic = MAGIC_SIG;
 	fill_time(&p.time);
 	return circular_put_packet(&p, NULL, 0);
 }
@@ -458,8 +453,6 @@ static inline void circular_put(const void *src, size_t count)
 		memcpy(Buffer, src + x, count - x);
 		BufWP = Buffer + count - x;
 	}
-
-	return;
 }
 
 static int circular_put_packet(struct rpldev_packet *p, const void *buf,
@@ -486,21 +479,20 @@ static int circular_put_packet(struct rpldev_packet *p, const void *buf,
 	return count;
 }
 
-static void fill_time(struct timeval *tv)
+static void fill_time(struct rpltime *x)
 {
-	uniqtime(tv);
+	struct timeval tv;
+	uniqtime(&tv);
 
-	if (sizeof(tv->tv_sec) == sizeof(uint32_t))
-		tv->tv_sec = LE_32(tv->tv_sec);
-	else if (sizeof(tv->tv_sec) == sizeof(uint64_t))
-		tv->tv_sec = LE_64(tv->tv_sec);
+	if (sizeof(tv.tv_sec) == sizeof(uint32_t))
+		x->tv_sec = LE_32(tv.tv_sec);
+	else if (sizeof(tv.tv_sec) == sizeof(uint64_t))
+		x->tv_sec = LE_64(tv.tv_sec);
 
-	if (sizeof(tv->tv_usec) == sizeof(uint32_t))
-		tv->tv_usec = LE_32(tv->tv_usec);
-	else if (sizeof(tv->tv_usec) == sizeof(uint64_t))
-		tv->tv_usec = LE_64(tv->tv_usec);
-
-	return;
+	if (sizeof(tv.tv_usec) == sizeof(uint32_t))
+		x->tv_usec = LE_32(tv.tv_usec);
+	else if (sizeof(tv.tv_usec) == sizeof(uint64_t))
+		x->tv_usec = LE_64(tv.tv_usec);
 }
 
 static inline unsigned int min_uint(unsigned int a, unsigned int b)
