@@ -1,12 +1,12 @@
 /*
- *	ttyrpld/user/infod.c
- *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2004 - 2008
+ *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2004 - 2009
  *
  *	This file is part of ttyrpld. ttyrpld is free software; you can
  *	redistribute it and/or modify it under the terms of the GNU
  *	Lesser General Public License as published by the Free Software
  *	Foundation; either version 2 or 3 of the License.
  */
+#include "config.h"
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -21,7 +21,7 @@
 #include <syslog.h>
 #include <unistd.h>
 
-#include <libHX/arbtree.h>
+#include <libHX/map.h>
 #include <libHX/string.h>
 #include "rpl_stdint.h"
 #include "dev.h"
@@ -64,7 +64,6 @@ void infod_init(void)
 	 */
 	chown(GOpt.infod_port, GOpt.user_id, 0);
 	chmod(GOpt.infod_port, S_IWUSR);
-	return;
 }
 
 void *infod_main(void *arg)
@@ -78,8 +77,7 @@ void *infod_main(void *arg)
 		pthread_t id;
 		int clfd;
 
-		if ((clfd = accept(Svfd, reinterpret_cast(void *,
-		    &remote), &sz)) < 0)
+		if ((clfd = accept(Svfd, const_cast(void *, &remote), &sz)) < 0)
 			continue;
 
 		pthread_create(&id, NULL, client_thread,
@@ -102,7 +100,7 @@ static void *client_thread(void *arg)
 
 		ret |= !RECEIVE_FULL(fd, &req, sizeof(unsigned char));
 		ret |= !RECEIVE_FULL(fd, &gint, sizeof(gint));
-		SWAB1(&gint);
+		gint = le32_to_cpu(gint);
 		if (ret || req == IFP_NONE)
 			break;
 
@@ -173,32 +171,30 @@ static void getinfo_text(uint32_t dev, int fd)
 	}
 
 	pthread_mutex_unlock(&Ttys_lock);
-	return;
 }
 
 static void getinfo_text_all(int fd)
 {
 	/* No device given, send info about RPLD and all ttys */
-	const struct HXbtree_node *node;
-	struct HXbtrav *trav;
+	const struct HXmap_node *node;
+	struct HXmap_trav *trav;
 
 	skprintf(fd,
 	"=========================================================="
 	"================\n"
-	"  (bytes)     %9llu %9llu ttyrpld " TTYRPLD_VERSION "/" ARCH "\n"
+	"  (bytes)     %9llu %9llu ttyrpld " PACKAGE_VERSION "/" ARCH "\n"
 	"  (packets)   %9lu %9lu\n"
 	"----------------------------------------------------------"
 	"----------------\n",
 	Stats.in, Stats.out, Stats.read, Stats.write);
 
-	trav = HXbtrav_init(Ttys);
-	while ((node = HXbtraverse(trav)) != NULL)
+	trav = HXmap_travinit(Ttys, HXMAP_DTRAV);
+	while ((node = HXmap_traverse(trav)) != NULL)
 		getinfo_text_one(fd, node->data);
 
 	skprintf(fd, "-----------------------------------------------"
 	"---------------------------\n");
-	HXbtrav_free(trav);
-	return;
+	HXmap_travfree(trav);
 }
 
 static void getinfo_text_one(int fd, struct tty *tty)
@@ -211,7 +207,6 @@ static void getinfo_text_one(int fd, struct tty *tty)
 	}
 	skprintf(fd, "%c %-11s %9lu %9lu %s\n", status, tty->sdev, tty->in,
 	         tty->out, basename_pp(tty->log, GOpt.ofmt));
-	return;
 }
 
 static void getinfo_bin(uint32_t dev, int fd)
@@ -229,26 +224,24 @@ static void getinfo_bin(uint32_t dev, int fd)
 	}
 
 	pthread_mutex_unlock(&Ttys_lock);
-	return;
 }
 
 static void getinfo_bin_all(int fd)
 {
 	/* No device given, send info about RPLD and all ttys */
-	const struct HXbtree_node *node;
-	struct HXbtrav *trav;
+	const struct HXmap_node *node;
+	struct HXmap_trav *trav;
 
-	skprintf(fd, "ttyrpld " TTYRPLD_VERSION "\n" "format 4\n"
+	skprintf(fd, "ttyrpld " PACKAGE_VERSION "\n" "format 4\n"
 	         "%lu %lu %lu %lu %llu %llu %lu\n",
 	         Stats.open, Stats.lclose, Stats.read, Stats.write,
 	         Stats.in, Stats.out, Stats.badpack);
 
-	trav = HXbtrav_init(Ttys);
-	while ((node = HXbtraverse(trav)) != NULL)
+	trav = HXmap_travinit(Ttys, HXMAP_DTRAV);
+	while ((node = HXmap_traverse(trav)) != NULL)
 		getinfo_bin_one(fd, node->data);
 
-	HXbtrav_free(trav);
-	return;
+	HXmap_travfree(trav);
 }
 
 static void getinfo_bin_one(int fd, struct tty *tty)
@@ -261,7 +254,6 @@ static void getinfo_bin_one(int fd, struct tty *tty)
 	}
 	skprintf(fd, "%c %ld %ld %s %lu %lu %s\n", status,
 	         KD26_PARTS(tty->dev), tty->sdev, tty->in, tty->out, tty->log);
-	return;
 }
 
 static void set_session_status(uint32_t dev, int req)
@@ -276,23 +268,22 @@ static void set_session_status(uint32_t dev, int req)
 
 	tty->status = req;
 	pthread_mutex_unlock(&Ttys_lock);
-	return;
 }
 
 static void zero_counters(uint32_t dev)
 {
 	pthread_mutex_lock(&Ttys_lock);
 	if (dev == 0) {
-		struct HXbtrav *trav = HXbtrav_init(Ttys);
-		const struct HXbtree_node *node;
+		struct HXmap_trav *trav = HXmap_travinit(Ttys, HXMAP_DTRAV);
+		const struct HXmap_node *node;
 
-		while ((node = HXbtraverse(trav)) != NULL) {
+		while ((node = HXmap_traverse(trav)) != NULL) {
 			struct tty *tty = node->data;
 			tty->in = tty->out = 0;
 		}
 
 		memset(&Stats, 0, sizeof(Stats));
-		HXbtrav_free(trav);
+		HXmap_travfree(trav);
 	} else {
 		struct tty *tty;
 		if ((tty = get_tty(dev, false)) == NULL) {
@@ -302,7 +293,6 @@ static void zero_counters(uint32_t dev)
 		tty->in = tty->out = 0;
 	}
 	pthread_mutex_unlock(&Ttys_lock);
-	return;
 }
 
 //-----------------------------------------------------------------------------
@@ -326,7 +316,6 @@ static void block_signals(void)
 	sigset_t set;
 	sigfillset(&set);
 	pthread_sigmask(SIG_BLOCK, &set, NULL);
-	return;
 }
 
 static int skprintf(int fd, const char *fmt, ...)
@@ -336,8 +325,8 @@ static int skprintf(int fd, const char *fmt, ...)
 	uint32_t le, s;
 
 	va_start(argp, fmt);
-	le = s = vsnprintf(buf, sizeof(buf), fmt, argp);
-	SWAB1(&le);
+	s  = vsnprintf(buf, sizeof(buf), fmt, argp);
+	le = cpu_to_le32(s);
 	send_wait(fd, &le, sizeof(uint32_t));
 	send_wait(fd, buf, s);
 
@@ -356,7 +345,7 @@ static int unix_server(const char *port)
 	unlink(sk.sun_path);
 
 	if ((fd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0 ||
-	    bind(fd, reinterpret_cast(struct sockaddr *, &sk),
+	    bind(fd, static_cast(void *, &sk),
 	    sizeof(struct sockaddr_un)) < 0 ||
 	    listen(fd, SOMAXCONN) < 0) {
 		close(fd);
@@ -365,5 +354,3 @@ static int unix_server(const char *port)
 
 	return fd;
 }
-
-//=============================================================================
