@@ -1,6 +1,5 @@
 /*
- *	ttyrpld/user/lib.c
- *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2004 - 2008
+ *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2004 - 2009
  *
  *	This file is part of ttyrpld. ttyrpld is free software; you can
  *	redistribute it and/or modify it under the terms of the GNU
@@ -48,7 +47,7 @@ void load_locale(const char *exe)
 		hxmc_t *base = HXmc_strinit(exe), *de;
 		HXmc_trunc(&base, HX_strrcspn(base, "/"));
 		HXmc_strcat(&base, "/locale");
-		de = HXmc_dup(base);
+		de = HXmc_meminit(base, HXmc_length(base));
 		HXmc_strcat(&de, "/de/LC_MESSAGES/ttyrpld.mo");
 		if (stat(de, &sb) == 0) {
 			bindtextdomain("ttyrpld", base);
@@ -68,7 +67,6 @@ void load_locale(const char *exe)
 	}
 
 	textdomain("ttyrpld");
-	return;
 }
 
 ssize_t read_through(int in, int out, size_t count)
@@ -171,19 +169,3 @@ off_t G_skip(int fd, off_t offset, int do_wait)
 
 	return 0;
 }
-
-void swab_be(void *srcp, size_t count)
-{
-	char *movp = srcp, x;
-
-	while (count) {
-		x = *movp++;
-		movp[0] = movp[1];
-		*++movp = x;
-		count -= 2;
-	}
-
-	return;
-}
-
-//=============================================================================
