@@ -1,17 +1,18 @@
 /*
- *	ttyrpld/user/replay.c
- *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2004 - 2008
+ *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2004 - 2009
  *
  *	This file is part of ttyrpld. ttyrpld is free software; you can
  *	redistribute it and/or modify it under the terms of the GNU
  *	Lesser General Public License as published by the Free Software
  *	Foundation; either version 2 or 3 of the License.
  */
+#include "config.h"
 #include <sys/stat.h>
 #include <sys/time.h> /* µsec def, gettimeofday() */
 #include <sys/types.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <limits.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -64,7 +65,8 @@ static void e_proc(int, struct rpldsk_packet *, struct pctrl_info *, char *,
 	struct rpltime *, long *);
 
 static unsigned long calc_ovcorr(unsigned long, int);
-static bool find_next_packet(int, const struct pctrl_info *);
+static bool find_next_packet(struct rpldsk_packet *, int,
+	const struct pctrl_info *);
 static bool get_options(int *, const char ***);
 static void getopt_jump(const struct HXoptcb *);
 static void getopt_msec(const struct HXoptcb *);
@@ -88,7 +90,7 @@ int main(int argc, const char **argv)
 	if (!get_options(&argc, &argv))
 		return EXIT_FAILURE;
 
-	fprintf(stderr, "> ttyreplay " TTYRPLD_VERSION "\n");
+	fprintf(stderr, "> ttyreplay " PACKAGE_VERSION "\n");
 	fprintf(stderr, _(
 	        "This program comes with ABSOLUTELY NO WARRANTY; it is free software and you\n"
 	        "you are welcome to redistribute it under certain conditions; for details see\n"
@@ -171,53 +173,51 @@ static int replay_file(int fd, const char *name)
 	while ((ret = read_waitfm(fd, &packet,
 	    sizeof(struct rpldsk_packet), &ps)) == sizeof(struct rpldsk_packet))
 	{
-		if (packet.magic != MAGIC_SIG) {
-			fprintf(stderr, _("\n" "<Packet inconsistency! "
+ redo:
+		packet.evmagic.n = be32_to_cpu(packet.evmagic.n);
+		if ((packet.evmagic.n & RPLEVT_MASK) != RPLEVT_NONE) {
+			fprintf(stderr, _("\n" "<Stream inconsistency! "
 			        "Trying to find next valid packet.>\n"));
 			tick = 0;
-			/*
-			 * If read() in find_next_packet() generates an error,
-			 * it will be catched when the condition is
-			 * re-evaulated upon continue.
-			 */
-			find_next_packet(fd, &ps);
-			continue;
+			if (!find_next_packet(&packet, fd, &ps))
+				continue;
+			goto redo;
 		}
 
-		SWAB1(&packet.size);
-		SWAB1(&packet.time.tv_sec);
-		SWAB1(&packet.time.tv_usec);
+		packet.size = le32_to_cpu(packet.size);
+		packet.time.tv_sec  = le64_to_cpu(packet.time.tv_sec);
+		packet.time.tv_usec = le32_to_cpu(packet.time.tv_usec);
 
-		switch (packet.event) {
-		case EVT_WRITE:
+		switch (packet.evmagic.n) {
+		case RPLEVT_WRITE:
 			e_proc(fd, &packet, &ps, &tick, &stamp, &skew);
 			break;
-		case EVT_ID_PROG:
+		case RPLEVT_ID_PROG:
 			fprintf(stderr, "\e[1;37;41m%s ", _("Created using")); /* ] */
 			read_through(fd, STDERR_FILENO, packet.size);
 			fprintf(stderr, "\e[0m\n"); /* ] */
 			break;
-		case EVT_ID_DEVPATH:
+		case RPLEVT_ID_DEVPATH:
 			fprintf(stderr, "\e[1;37;41m%s: ", _("Device path")); /* ] */
 			read_through(fd, STDERR_FILENO, packet.size);
 			fprintf(stderr, "\e[0m\n"); /* ] */
 			break;
-		case EVT_ID_TIME:
+		case RPLEVT_ID_TIME:
 			fprintf(stderr, "\e[1;37;41m%s ", _("Recorded on")); /* ] */
 			read_through(fd, STDERR_FILENO, packet.size);
 			fprintf(stderr, "\e[0m\n"); /* ] */
 			break;
-		case EVT_ID_USER:
+		case RPLEVT_ID_USER:
 			fprintf(stderr, "\e[1;37;41m%s: ", _("User")); /* ] */
 			read_through(fd, STDERR_FILENO, packet.size);
 			fprintf(stderr, "\e[0m\n"); /* ] */
 			break;
-		case EVT_LCLOSE:
+		case RPLEVT_LCLOSE:
 			fprintf(stderr, "\n\e[1;37;42m%s\e[0m\n", /* ]] */
 			        _("<tty device has been closed>"));
 			read_nullfm(fd, packet.size);
 			break;
-		case EVT_READ:
+		case RPLEVT_READ:
 			if (ps.echo) {
 				printf("\e[1;37;45m"); /* ] */
 				e_proc(fd, &packet, &ps, &tick, &stamp, &skew);
@@ -319,7 +319,6 @@ static void e_proc(int fd, struct rpldsk_packet *p, struct pctrl_info *i,
 
 		write(STDOUT_FILENO, buf, strlen(buf));
 	}
-	return;
 }
 
 //-----------------------------------------------------------------------------
@@ -345,128 +344,32 @@ static unsigned long calc_ovcorr(unsigned long ad, int rd)
 	return av;
 }
 
-static bool find_next_packet(int fd, const struct pctrl_info *ps)
+static bool find_next_packet(struct rpldsk_packet *packet, int fd,
+    const struct pctrl_info *ps)
 {
-	/* Interesting, there is more algorithmic code than simple
-	code in ttyreplay. Outstanding. */
-#define LZ           sizeof(struct rpldsk_packet)
-#define BZ           (2 * LZ)
-#define MAGIC_OFFSET offsetof(struct rpldsk_packet, magic)
-	char buf[BZ];
-	struct rpldsk_packet *packet = static_cast(void *, buf);
-	size_t s;
-	unsigned int ok = 0;
+#define LZ           HXsizeof_member(struct rpldsk_packet, evmagic.n)
+	unsigned int rem;
+	uint32_t magic;
+	void *start;
 
-	if (read_waitfm(fd, buf, BZ, ps) < BZ)
+	if (read_waitfm(fd, packet, sizeof(packet->evmagic), ps) <
+	    sizeof(packet->evmagic))
 		return false;
 
-	while (true) {
-		char *ptr;
-
-		/*
-		 * Indeed, the many read() calls get more and more data from
-		 * the stream without displaying it. Anyway, if we found a way
-		 * into this function, there is a reason to it.
-		 */
-
-		if ((ptr = memchr(buf, MAGIC_SIG, BZ)) != NULL &&
-		    ptr - buf >= MAGIC_OFFSET) {
-			/*
-			 * A magic byte has been found and the packet start is
-			 * complete
-			 */
-			char *ctx = reinterpret_cast(char *,
-			            containerof(ptr, struct rpldsk_packet, magic));
-			if (ctx != buf) {
-				size_t cnt = buf + BZ - ctx;
-				ctx = memmove(buf, ctx, cnt);
-				if (read_waitfm(fd, buf + cnt, BZ - cnt, ps) < BZ - cnt)
-					return false;
-			}
-		} else if (ptr != NULL) {
-			/*
-			 * Magic byte, but of no use. Discard it, and fill up
-			 * with new data from the descriptor.
-			 */
-			size_t cnt = buf + BZ - ptr - 1;
-			memmove(buf, ptr + 1, cnt);
-			if (read_waitfm(fd, buf + cnt, BZ - cnt, ps) < BZ - cnt)
-				return false;
-			continue;
-		} else {
-			/*
-			 * No magic byte, but since it might be just the next
-			 * byte in the stream, only read LZ bytes.
-			 */
-			memmove(buf, buf + BZ - LZ, BZ - LZ);
-			if (read_waitfm(fd, buf + LZ, BZ - LZ, ps) < BZ - LZ)
-				return false;
-			continue;
-		}
-
-		s = packet->size;
-		if (s > 4096) {
-			/*
-			 * The default tty buffer size is 4096, and any size
-			 * above this is questionable at all. Start with a new
-			 * slate.
-			 */
-			ok = 0;
-			memmove(buf, buf + LZ, BZ - LZ);
-			if (read_waitfm(fd, buf, BZ, ps) < BZ)
-				return false;
-			continue;
-		}
-
-		if (s < LZ) {
-			memmove(buf, buf + LZ + s, BZ - LZ - s);
-			if (read_waitfm(fd, buf + BZ - LZ - s, LZ + s, ps) < LZ + s)
-				return false;
-		} else {
-			/*
-			 * There is no header (according to .size) in our buffer, so we
-			 * can blindly munge lots of data.
-			 */
-			if (!read_nullfm(fd, s - LZ) || read_waitfm(fd, buf, BZ, ps) < BZ)
-				return false;
-		}
-
-		if ((ptr = memchr(buf, MAGIC_SIG, MAGIC_OFFSET + 1)) == NULL ||
-		    ptr - buf != MAGIC_OFFSET) {
-			/*
-			 * If the size field does not match up with the next
-			 * magic byte, drop it all.
-			 */
-			ok = 0;
-			continue;
-		}
-
-		if (++ok >= 2)
-			break;
+	magic = be32_to_cpu(packet->evmagic.n);
+	while ((magic & RPLEVT_MASK) == RPLEVT_NONE) {
+		packet->evmagic.n = magic = cpu_to_be32(magic << CHAR_BIT);
+		if (!read_waitfm(fd, &packet->evmagic.m[
+		    ARRAY_SIZE(packet->evmagic.m)-1], 1, ps) != 1)
+			return false;
+		magic = be32_to_cpu(magic);
 	}
 
 	fprintf(stderr, _("\n" "<Found packet boundary>\n"));
-
-	/* Finally adjust the read pointer to a packet boundary */
-	if ((s = packet->size) < 16) {
-		/*
-		 * Mmhkay, there is another header other than the current
-		 * (packet) in the buffer. Crap, another one gone.
-		 */
-		memmove(buf, buf + LZ + s, BZ - LZ - s);
-		if (read_waitfm(fd, buf + BZ - LZ - s, s, ps) < s ||
-		    !read_nullfm(fd, packet->size))
-			return false;
-	} else {
-		/* Just subtract what we have already read into the buffer */
-		if (!read_nullfm(fd, s - LZ))
-			return false;
-	}
-
-	return true;
-#undef BZ
+	start = static_cast(void *, packet) + sizeof(packet->evmagic);
+	rem   = sizeof(*packet) - sizeof(packet->evmagic);
+	return read_waitfm(fd, start, rem, ps) == rem;
 #undef LZ
-#undef MAGIC_OFFSET
 }
 
 static bool get_options(int *argc, const char ***argv)
@@ -486,7 +389,7 @@ static bool get_options(int *argc, const char ***argv)
         {.sh = 'f', .type = HXTYPE_VAL, .ptr = &Opt.follow, .val = FM_CATCHUP,
          .help = _("Catch-up follow mode (play file, switch to live feed on EOF)")},
         {.sh = 'j', .type = HXTYPE_LONG, .ptr = &Opt.skval, .cb = getopt_skip,
-         .help = _("Skip the given number of EVT_WRITE packets")},
+         .help = _("Skip the given number of RPLEVT_WRITE packets")},
         {.sh = 'm', .type = HXTYPE_LONG, .cb = getopt_msec,
          .help = _("Maximum delay to wait between packets"), .htyp = _("msec")},
         {.sh = 't', .type = HXTYPE_NONE | HXOPT_INC, .ptr = &Opt.showtime,
@@ -521,19 +424,16 @@ static void getopt_jump(const struct HXoptcb *cbi)
 	Opt.skval = (strtol(sec, NULL, 0) + 60 * strtol(min, NULL, 0) +
 	            3600 * strtol(hr, NULL, 0)) * 1000;
 	free(origtmp);
-	return;
 }
 
 static void getopt_msec(const struct HXoptcb *cbi)
 {
 	Opt.maxdelay = cbi->data_long * 1000;
-	return;
 }
 
 static void getopt_skip(const struct HXoptcb *cbi)
 {
 	Opt.sktype = PCTRL_SKPACK;
-	return;
 }
 
 static inline ssize_t read_nullfm(int fd, size_t count)
@@ -578,6 +478,7 @@ static ssize_t read_waitfm(int fd, void *buf, size_t count,
 
 static bool seek_to_end(int fd, const struct pctrl_info *ps)
 {
+	struct rpldsk_packet dummy;
 	off_t a, b;
 
 	a = lseek(fd, 0, SEEK_END);
@@ -608,7 +509,7 @@ static bool seek_to_end(int fd, const struct pctrl_info *ps)
 		return false;
 	}
 
-	return find_next_packet(fd, ps);
+	return find_next_packet(&dummy, fd, ps);
 }
 
 /*  usleep_ovcorr
@@ -706,7 +607,6 @@ static void usleep_ovcorr(struct timeval *req, long *skew)
 	         (stop.tv_usec - start.tv_usec);
 	over   = dur - req_usec;
 	*skew += over;
-	return;
 }
 
 /*
@@ -731,8 +631,6 @@ static void tv_delta(const struct rpltime *past, const struct rpltime *now,
 		dest->tv_sec  = sec;
 		dest->tv_usec = acc;
 	}
-
-	return;
 }
 
 static inline unsigned long long tv2usec(const struct timeval *req)
@@ -744,7 +642,4 @@ static inline void usec2tv(unsigned long long usec, struct timeval *req)
 {
 	req->tv_sec  = usec / MICROSECOND;
 	req->tv_usec = usec % MICROSECOND;
-	return;
 }
-
-//=============================================================================
