@@ -1,6 +1,5 @@
 /*
- *	ttyrpld/user/rplcvt.c
- *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2007 - 2008
+ *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2007 - 2009
  *
  *	This file is part of ttyrpld. ttyrpld is free software; you can
  *	redistribute it and/or modify it under the terms of the GNU
@@ -21,7 +20,29 @@
 #include "../include/rpl_packet.h"
 #include "lib.h"
 
+enum {
+	EVT2_NONE       = 0x00,
+	EVT2_OPEN       = 0x01,
+	EVT2_READ       = 0x02,
+	EVT2_WRITE      = 0x03,
+	EVT2_MAGIC      = 0x4A,
+	EVT2_LCLOSE     = 0x64,
+	EVT2_ID_PROG    = 0xF0,
+	EVT2_ID_DEVPATH = 0xF1,
+	EVT2_ID_TIME    = 0xF2,
+	EVT2_ID_USER    = 0xF3,
+	EVT2_max,
+	MAGIC_SIG2      = 0xEE,
+};
+
 /* Definitions */
+enum {
+	F_NONE,
+	F_RPLDSK1_32,
+	F_RPLDSK1_64,
+	F_RPLDSK2,
+};
+
 struct rpldsk1_packet32 {
 	uint16_t size;
 	uint8_t event, magic;
@@ -40,15 +61,49 @@ struct rpldsk1_packet64 {
 	} time;
 };
 
+struct rpldsk2_packet {
+	uint16_t size;
+	uint8_t event, magic;
+	struct rpltime time;
+} __PACKED;
+
 /* Functions */
 static bool rplcvt_get_options(int *, const char ***);
-static void rplcvt_32(int);
-static void rplcvt_64(int);
+static void rplcvt1_32(int);
+static void rplcvt1_64(int);
 
 /* Variables */
-static unsigned int rplcvt_bitness;
+static char *rplcvt_format;
+static const uint32_t ev2_to_ev3[] = {
+	[EVT2_NONE]       = cpu_to_be32(RPLEVT_NONE),
+	[EVT2_OPEN]       = cpu_to_be32(RPLEVT_OPEN),
+	[EVT2_READ]       = cpu_to_be32(RPLEVT_READ),
+	[EVT2_WRITE]      = cpu_to_be32(RPLEVT_WRITE),
+	[EVT2_MAGIC]      = cpu_to_be32(RPLEVT_NONE),
+	[EVT2_LCLOSE]     = cpu_to_be32(RPLEVT_LCLOSE),
+	[EVT2_ID_PROG]    = cpu_to_be32(RPLEVT_ID_PROG),
+	[EVT2_ID_DEVPATH] = cpu_to_be32(RPLEVT_ID_DEVPATH),
+	[EVT2_ID_TIME]    = cpu_to_be32(RPLEVT_ID_TIME),
+	[EVT2_ID_USER]    = cpu_to_be32(RPLEVT_ID_USER),
+};
 
 //-----------------------------------------------------------------------------
+static void rplcvt2(int fd)
+{
+	struct rpldsk2_packet old;
+	struct rpldsk_packet new;
+	ssize_t ret;
+
+	while ((ret = read(fd, &old, sizeof(old))) == sizeof(old)) {
+		new.evmagic.n    = ev2_to_ev3[old.event];
+		new.size         = old.size;
+		new.time.tv_sec  = old.time.tv_sec;
+		new.time.tv_usec = old.time.tv_usec;
+		write(STDOUT_FILENO, &new, sizeof(new));
+		read_through(fd, STDOUT_FILENO, new.size);
+	}
+}
+
 int main(int argc, const char **argv)
 {
 	int fd;
@@ -70,13 +125,14 @@ int main(int argc, const char **argv)
 		}
 	}
 
-	if (rplcvt_bitness == 32) {
-		rplcvt_32(fd);
-	} else if (rplcvt_bitness == 64) {
-		rplcvt_64(fd);
+	if (strcmp(rplcvt_format, "2") == 0) {
+		rplcvt2(fd);
+	} else if (strcmp(rplcvt_format, "1@32") == 0) {
+		rplcvt1_32(fd);
+	} else if (strcmp(rplcvt_format, "1@64") == 0) {
+		rplcvt1_64(fd);
 	} else {
-		fprintf(stderr, "%s: You need to specify either --32 "
-		        "or --64\n", *argv);
+		fprintf(stderr, "%s: Unknown format specified with -F\n", *argv);
 		return EXIT_FAILURE;
 	}
 
@@ -89,26 +145,29 @@ int main(int argc, const char **argv)
 static bool rplcvt_get_options(int *argc, const char ***argv)
 {
 	struct HXoption options_table[] = {
-		{.ln = "32", .type = HXTYPE_VAL, .ptr = &rplcvt_bitness,
-		 .val = 32, .help = "Select 32-bit decoder"},
-		{.ln = "64", .type = HXTYPE_VAL, .ptr = &rplcvt_bitness,
-		 .val = 64, .help = "Select 64-bit decoder"},
+		{.sh = 'F', .type = HXTYPE_STRING, .ptr = &rplcvt_format,
+		 .help = "Input format (\"1@32\", \"1@64\", \"2\")"},
 		HXOPT_AUTOHELP,
 		HXOPT_TABLEEND,
 	};
-	return HX_getopt(options_table, argc, argv, HXOPT_USAGEONERR) > 0;
+	if (HX_getopt(options_table, argc, argv, HXOPT_USAGEONERR) <= 0)
+		return false;
+	if (rplcvt_format == NULL) {
+		fprintf(stderr, "The -F option is mandatory\n");
+		return false;
+	}
+	return true;
 }
 
-static void rplcvt_32(int fd)
+static void rplcvt1_32(int fd)
 {
 	struct rpldsk1_packet32 old;
 	struct rpldsk_packet new;
 	ssize_t ret;
 
 	while ((ret = read(fd, &old, sizeof(old))) == sizeof(old)) {
+		new.evmagic.n    = ev2_to_ev3[old.event];
 		new.size         = old.size;
-		new.event        = old.event;
-		new.magic        = old.magic;
 		new.time.tv_sec  = old.time.tv_sec;
 		new.time.tv_usec = old.time.tv_usec;
 		write(STDOUT_FILENO, &new, sizeof(new));
@@ -116,19 +175,19 @@ static void rplcvt_32(int fd)
 	}
 }
 
-static void rplcvt_64(int fd)
+static void rplcvt1_64(int fd)
 {
 	struct rpldsk1_packet64 old;
 	struct rpldsk_packet new;
 	ssize_t ret;
 
 	while ((ret = read(fd, &old, sizeof(old))) == sizeof(old)) {
+		new.evmagic.n    = ev2_to_ev3[old.event];
 		new.size         = old.size;
-		new.event        = old.event;
-		new.magic        = old.magic;
 		new.time.tv_sec  = old.time.tv_sec;
 		new.time.tv_usec = old.time.tv_usec;
 		write(STDOUT_FILENO, &new, sizeof(new));
 		read_through(fd, STDOUT_FILENO, new.size);
 	}
 }
+
