@@ -1,6 +1,5 @@
 /*
- *	ttyrpld/user/rpld.c
- *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2004 - 2008
+ *	Copyright © Jan Engelhardt <jengelh [at] medozas de>, 2004 - 2009
  *
  *	This file is part of ttyrpld. ttyrpld is free software; you can
  *	redistribute it and/or modify it under the terms of the GNU
@@ -12,9 +11,6 @@
 #include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#ifdef HAVE_ALLOCA
-#	include <alloca.h>
-#endif
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -30,7 +26,7 @@
 #include <time.h>
 #include <unistd.h>
 
-#include <libHX/arbtree.h>
+#include <libHX/map.h>
 #include <libHX/misc.h>
 #include <libHX/option.h>
 #include <libHX/string.h>
@@ -51,6 +47,16 @@ enum {
 	C_MAX,
 };
 
+/**
+ * @string:	initial escape code to match
+ * @striplen:	function to determine exact length to strip
+ */
+struct filter {
+	const char *string;
+	unsigned int (*striplen)(const char *, unsigned int);
+	unsigned int length;
+};
+
 /* Functions */
 static void mainloop(int);
 
@@ -59,7 +65,7 @@ static bool packet_process(struct rpldev_packet *, struct tty *, int);
 
 static void evt_open(struct rpldev_packet *, struct tty *, int);
 static void log_open(struct tty *);
-static void log_write(struct rpldev_packet *, struct tty *, int);
+static void log_write(struct rpldev_packet *, struct tty *, int, unsigned int);
 
 static int check_parent_directory(const hxmc_t *);
 static void fill_info(struct tty *, const char *);
@@ -87,7 +93,7 @@ static struct {
 	char *device;
 	long bsize;
 	int max_fd;
-	unsigned int dolog, infod_start;
+	unsigned int dolog, infod_start, filter_esc;
 	bool _running;
 } Opt = {
 	._running    = true,
@@ -96,6 +102,7 @@ static struct {
 	.dolog       = true,
 	.infod_start = false,
 	.max_fd      = -1, /* use system default */
+	.filter_esc  = true, /* filter deadly escapes */
 };
 
 //-----------------------------------------------------------------------------
@@ -129,7 +136,7 @@ int main(int argc, const char **argv)
 	memset(&Stats, 0, sizeof(Stats));
 
 	if (GOpt.verbose) {
-		printf("# rpld " TTYRPLD_VERSION "\n");
+		printf("# rpld " PACKAGE_VERSION "\n");
 		printf(_(
 		       "This program comes with ABSOLUTELY NO WARRANTY; it is free software and you\n"
 		       "you are welcome to redistribute it under certain conditions; for details see\n"
@@ -138,7 +145,7 @@ int main(int argc, const char **argv)
 		printf("\n");
 	}
 
-	if ((Ttys = HXbtree_init(HXBT_MAP | HXBT_ICMP)) == NULL) {
+	if ((Ttys = HXmap_init(HXMAPT_ORDERED, 0)) == NULL) {
 		perror("Ttys = HXbtree_init()");
 		return EXIT_FAILURE;
 	}
@@ -192,17 +199,18 @@ static void mainloop(int fd)
 		break;
 	}
 
-	SWAB1(&packet.dev);
-	SWAB1(&packet.size);
-	SWAB1(&packet.time.tv_sec);
-	SWAB1(&packet.time.tv_usec);
+	packet.evmagic.n = be32_to_cpu(packet.evmagic.n);
+	packet.dev  = le32_to_cpu(packet.dev);
+	packet.size = le32_to_cpu(packet.size);
+	packet.time.tv_sec  = le64_to_cpu(packet.time.tv_sec);
+	packet.time.tv_usec = le32_to_cpu(packet.time.tv_usec);
 
-	if (packet.magic != MAGIC_SIG) {
+	if ((packet.evmagic.n & RPLEVT_MASK) != RPLEVT_NONE) {
 		++Stats.badpack;
 		if (rate_limit(C_BADPACKET, 2))
 			notify(LOG_WARNING,
-			       _("Bogus packet (magic is 0x%02X)!\n"),
-			       packet.magic);
+			       _("Bogus packet (evmagic is 0x%08x)!\n"),
+			       packet.evmagic.n);
 		continue;
 	}
 
@@ -227,47 +235,47 @@ static void mainloop(int fd)
 
 	pthread_mutex_unlock(&Ttys_lock);
     }
-
-    return;
 }
 
 //-----------------------------------------------------------------------------
 static bool packet_preprox(struct rpldev_packet *packet)
 {
+#define E(x) ((x) & ~RPLEVT_MASK)
 	static unsigned long *const tab[] = {
-		[EVT_OPEN]   = &Stats.open,
-		[EVT_READ]   = &Stats.read,
-		[EVT_WRITE]  = &Stats.write,
-		[EVT_LCLOSE] = &Stats.lclose,
-		[EVT_max]    = NULL,
+		[E(RPLEVT_OPEN)]   = &Stats.open,
+		[E(RPLEVT_READ)]   = &Stats.read,
+		[E(RPLEVT_WRITE)]  = &Stats.write,
+		[E(RPLEVT_LCLOSE)] = &Stats.lclose,
+		[E(RPLEVT_max)]    = NULL,
 	};
 
-	if (packet->event < EVT_max && tab[packet->event] != NULL)
-		++*tab[packet->event];
+	if (packet->evmagic.n < RPLEVT_max && tab[E(packet->evmagic.n)] != NULL)
+		++*tab[E(packet->evmagic.n)];
 
 	/* General packet classification (first stage drop) */
-	switch (packet->event) {
+	switch (packet->evmagic.n) {
 	/* These will be processed */
-	case EVT_OPEN:
-	case EVT_LCLOSE:
+	case RPLEVT_OPEN:
+	case RPLEVT_LCLOSE:
 		break;
 
 	/* The following roll their own and will also be processed... */
-	case EVT_READ:
+	case RPLEVT_READ:
 		Stats.in += packet->size;
 		break;
-	case EVT_WRITE:
+	case RPLEVT_WRITE:
 		Stats.out += packet->size;
 		break;
 	default:
 		if (rate_limit(C_PKTTYPE, 2))
 			notify(LOG_WARNING,
-			       _("Unknown packet type 0x%02X\n"),
-			       packet->event);
+			       _("Unknown packet type 0x%08x\n"),
+			       packet->evmagic.n);
 		return false;
 	} /* switch */
 
 	return true;
+#undef E
 }
 
 static bool packet_process(struct rpldev_packet *packet,
@@ -279,36 +287,36 @@ static bool packet_process(struct rpldev_packet *packet,
 	}
 
 	if (tty->status != IFP_ACTIVATE) {
-		switch (packet->event) {
-			case EVT_READ:
+		switch (packet->evmagic.n) {
+			case RPLEVT_READ:
 				tty->in += packet->size;
 				break;
-			case EVT_WRITE:
+			case RPLEVT_WRITE:
 				tty->out += packet->size;
 				break;
 		}
 		return false;
 	}
 
-	switch (packet->event) {
-		case EVT_OPEN:
+	switch (packet->evmagic.n) {
+		case RPLEVT_OPEN:
 			evt_open(packet, tty, fd);
 			return true;
-		case EVT_READ:
+		case RPLEVT_READ:
 			tty->in += packet->size;
-			log_write(packet, tty, fd);
+			log_write(packet, tty, fd, packet->evmagic.n);
 			return true;
-		case EVT_WRITE:
+		case RPLEVT_WRITE:
 			tty->out += packet->size;
-			log_write(packet, tty, fd);
+			log_write(packet, tty, fd, packet->evmagic.n);
 			return true;
-		case EVT_LCLOSE:
+		case RPLEVT_LCLOSE:
 			log_close(tty);
 			break;
 		default:
 			notify(LOG_ERR, _("Should never get here! (%s:%d) "
-			       "Forgot to code something? (event=%d)\n"),
-			       __FILE__, __LINE__, packet->event);
+			       "Forgot to code something? (event=%x)\n"),
+			       __FILE__, __LINE__, packet->evmagic.n);
 			break;
 	}
 
@@ -326,10 +334,16 @@ static void evt_open(struct rpldev_packet *packet, struct tty *tty, int fd)
 	 * - Find out about UID changes on the device and start a new logfile
 	 *   if owner has changed.
 	 */
-	char *sdev = alloca(packet->size + 1);
+	char *sdev;
 	bool owner_changed = false, fill_it = false;
 	struct stat sb;
 
+	sdev = malloc(packet->size + 1);
+	if (sdev == NULL) {
+		perror("malloc");
+		return;
+	}
+
 	read(fd, sdev, packet->size);
 	sdev[packet->size] = '\0';
 
@@ -346,15 +360,14 @@ static void evt_open(struct rpldev_packet *packet, struct tty *tty, int fd)
 
 	if (fill_it)
 		fill_info(tty, sdev);
+	free(sdev);
 	if (owner_changed)
 		log_open(tty);
-
-	return;
 }
 
 static void log_open(struct tty *tty)
 {
-	struct rpldsk_packet p = {.magic = MAGIC_SIG, .time = {-1, -1}};
+	struct rpldsk_packet p;
 	struct tm now_tm, *nowp;
 	char buf[MAXFNLEN];
 	time_t now_sec;
@@ -371,26 +384,20 @@ static void log_open(struct tty *tty)
 	tty->fd = open(tty->log, O_WRONLY | O_CREAT | O_APPEND,
 	          S_IRUSR | S_IWUSR);
 
-	/*
-	 * Add an optional magic packet for file(1) to recognize. (EVT_ID_PROG
-	 * may contain _anything_, while EVT_MAGIC is fixed. See
-	 * share/ttyrpld.magic.)
-	 */
-	HX_strlcpy(buf, "RPL2_50", sizeof(buf)); /* STRING FIXED */
-	p.event = EVT_MAGIC;
-	s = p.size = strlen(buf) + 1; /* include '\0' in stream */
-	SWAB1(&p.size);
+	/* Add an optional magic packet for file(1) to recognize. */
+	p.evmagic.n = cpu_to_be32(RPLEVT_NONE);
+	p.size = 0;
+	memset(&p.time, 0xFA, sizeof(p.time));
 	write(tty->fd, &p, sizeof(struct rpldsk_packet));
-	write(tty->fd, buf, s);
 
 	/*
 	 * Add an optional ident header to record the program and version which
 	 * this logfile was created with.
 	 */
-	HX_strlcpy(buf, "ttyrpld " TTYRPLD_VERSION, sizeof(buf));
-	p.event = EVT_ID_PROG;
-	s = p.size = strlen(buf) + 1;
-	SWAB1(&p.size);
+	HX_strlcpy(buf, "ttyrpld " PACKAGE_VERSION, sizeof(buf));
+	p.evmagic.n = cpu_to_be32(RPLEVT_ID_PROG);
+	s = strlen(buf) + 1;
+	p.size = cpu_to_le32(s);
 	write(tty->fd, &p, sizeof(struct rpldsk_packet));
 	write(tty->fd, buf, s);
 
@@ -398,68 +405,142 @@ static void log_open(struct tty *tty)
 	now_sec = time(NULL);
 	nowp = localtime_r(&now_sec, &now_tm);
 	strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S", nowp);
-	p.event = EVT_ID_TIME;
-	s = p.size = strlen(buf) + 1;
-	SWAB1(&p.size);
+	p.evmagic.n = cpu_to_be32(RPLEVT_ID_TIME);
+	s = strlen(buf) + 1;
+	p.size = cpu_to_le32(s);
 	write(tty->fd, &p, sizeof(struct rpldsk_packet));
 	write(tty->fd, buf, s);
 
 	/* ... and the full path name of the tty for reference. */
-	p.event = EVT_ID_DEVPATH;
-	s = p.size = strlen(tty->full_dev) + 1;
-	SWAB1(&p.size);
+	p.evmagic.n = cpu_to_be32(RPLEVT_ID_DEVPATH);
+	s = strlen(tty->full_dev) + 1;
+	p.size = cpu_to_le32(s);
 	write(tty->fd, &p, sizeof(struct rpldsk_packet));
 	write(tty->fd, tty->full_dev, s);
 
 	/* ... as well as the username (or UID) the tty belongs to */
 	if (getnamefromuid(tty->uid, buf, sizeof(buf)) == NULL)
 		snprintf(buf, sizeof(buf), "%ld", static_cast(long, tty->uid));
-	p.event = EVT_ID_USER;
-	s = p.size = strlen(buf) + 1;
-	SWAB1(&p.size);
+	p.evmagic.n = cpu_to_be32(RPLEVT_ID_USER);
+	s = strlen(buf) + 1;
+	p.size = cpu_to_le32(s);
 	write(tty->fd, &p, sizeof(struct rpldsk_packet));
 	write(tty->fd, buf, s);
-
-	return;
 }
 
-static void log_write(struct rpldev_packet *packet, struct tty *tty, int fd)
+static inline unsigned int min2(unsigned int a, unsigned int b)
 {
-	char *buffer = alloca(packet->size);
+	return (a < b) ? a : b;
+}
+
+static unsigned int rpld_filter(char *buffer, unsigned int bufsize,
+    const struct filter *f)
+{
+	unsigned int len, size;
+	char *p;
+
+	for (; f->string != NULL; ++f) {
+		size = bufsize;
+		p = buffer;
+		while ((p = memchr(p, *f->string, size)) != NULL) {
+			size = buffer + bufsize - p;
+			if (f->length > size ||
+			    memcmp(p, f->string, f->length) != 0) {
+				++p;
+				size = buffer + bufsize - p;
+				continue;
+			}
+			len = (f->striplen == NULL) ? f->length :
+			      f->striplen(p, size);
+			if (len > size)
+				abort();
+			memmove(p, p + len, size - len);
+			bufsize -= len;
+			size -= len;
+		}
+	}
+
+	return bufsize;
+}
+
+static unsigned int xterm_mouse_filter(const char *buffer, unsigned int size)
+{
+	return min2(size, 6);
+}
+
+static void log_write(struct rpldev_packet *packet, struct tty *tty, int fd,
+    unsigned int event)
+{
+	static const struct filter write_filters[] = {
+		/* VT100 identification request */
+		{"\033Z", NULL, 2},
+		/* X10 mouse reporting request */
+		{"\033[?9h", NULL, 5}, // ]
+		/* X11 mouse reporting request */
+		{"\033[?1000h", NULL, 8}, // ]
+		{NULL},
+	};
+	static const struct filter read_filters[] = {
+		/* Linux ident response */
+		{"\033[?6c", NULL, 5}, // ]
+		/* VT100 ident response */
+		{"\033[?1;2c", NULL, 7}, // ]
+		/* Xterm mouse codes */
+		{"\033[M", xterm_mouse_filter, 3}, // ]
+		{NULL},
+	};
+	char *buffer;
 	ssize_t have;
 
+	buffer = malloc(packet->size + 1);
+	if (buffer == NULL)
+		return;
 	if (tty->fd < 0)
 		log_open(tty);
 	if ((have = read(fd, buffer, packet->size)) <= 0)
-		return;
+		goto out;
 	if (have != packet->size)
 		packet->size = have;
+	buffer[have] = '\0';
 
-	SWAB1(&packet->size);
-	SWAB1(&packet->time.tv_sec);
-	SWAB1(&packet->time.tv_usec);
-	write(tty->fd, &packet->size, sizeof(struct rpldsk_packet));
+	/* flip it back */
+	packet->evmagic.n = cpu_to_be32(packet->evmagic.n);
+
+	packet->size = cpu_to_le32(packet->size);
+	packet->time.tv_sec  = cpu_to_le64(packet->time.tv_sec);
+	packet->time.tv_usec = cpu_to_le32(packet->time.tv_usec);
+	write(tty->fd, packet, sizeof(struct rpldsk_packet));
+	if (Opt.filter_esc) {
+		if (event == RPLEVT_WRITE)
+			have = rpld_filter(buffer, have, write_filters);
+		else if (event == RPLEVT_READ)
+			have = rpld_filter(buffer, have, read_filters);
+	}
 	write(tty->fd, buffer, have);
-	return;
+ out:
+	free(buffer);
 }
 
 //-----------------------------------------------------------------------------
 static int check_parent_directory(const hxmc_t *s)
 {
-	char *path = alloca(strlen(s) + 1), *p;
+	char *path, *p;
+	int ret;
 
-	strcpy(path, s);
-	if ((p = strrchr(path, '/')) == NULL)
+	if ((p = strrchr(s, '/')) == NULL)
 		/* Current dirctory, no more dir checks needed */
 		return 1;
 
-	*p = '\0';
-	return HX_mkdir(path); /* `mkdir -p` */
+	path = HX_strdup(s);
+	path[p-s] = '\0';
+	ret = HX_mkdir(path); /* `mkdir -p` */
+	free(path);
+	return ret;
 }
 
 static void fill_info(struct tty *tty, const char *aux_sdev)
 {
-	struct HXbtree *catalog;
+	struct HXformat_map *catalog;
 	char full_dev[MAXFNLEN], sdev[MAXFNLEN],
 		fmday[16], fmtime[16], user[64];
 	const char *pbase = NULL;
@@ -546,14 +627,13 @@ static void fill_info(struct tty *tty, const char *aux_sdev)
 	}
 
 	HXmc_free(tty->log);
-	catalog  = HXformat_init();
+	catalog = HXformat_init();
 	HXformat_add(catalog, "DATE", fmday,  HXTYPE_STRING);
 	HXformat_add(catalog, "TIME", fmtime, HXTYPE_STRING);
 	HXformat_add(catalog, "TTY",  sdev,   HXTYPE_STRING);
 	HXformat_add(catalog, "USER", user,   HXTYPE_STRING);
 	HXformat_aprintf(catalog, &tty->log, GOpt.ofmt);
 	HXformat_free(catalog);
-	return;
 }
 
 //-----------------------------------------------------------------------------
@@ -603,7 +683,6 @@ static void init_fdtable(rlim_t nfd)
 	if (setrlimit(RLIMIT_NOFILE, &rl) != 0)
 		fprintf(stderr, _("Warning: Could not increase fd table size "
 		        "to %d: %s\n"), (int)nfd, strerror(errno));
-	return;
 }
 
 static int init_sighandler(void)
@@ -644,7 +723,6 @@ static void sighandler_int(int s)
 	if (GOpt.verbose)
 		printf(_("\n" "Received SIGINT/SIGTERM, shutting down.\n"));
 	Opt._running = 0;
-	return;
 }
 
 static void sighandler_alrm(int s)
@@ -654,14 +732,12 @@ static void sighandler_alrm(int s)
 	fflush(stdout);
 	if (GOpt.verbose)
 		alarm(1);
-	return;
 }
 
 static void sighandler_pipe(int s)
 {
 	fprintf(stderr, _("\n" "[%d] Received SIGPIPE\n"),
 	        static_cast(int, getpid()));
-	return;
 }
 
 //-----------------------------------------------------------------------------
@@ -763,6 +839,8 @@ static bool get_options(int *argc, const char ***argv)
 	struct HXoption options_table[] = {
 	        {.sh = 'D', .type = HXTYPE_STRING, .ptr = &Opt.device,
 	         .help = _("Path to the RPL device"), .htyp = _("file")},
+	        {.sh = 'E', .type = HXTYPE_VAL, .ptr = &Opt.filter_esc,
+	         .val = false, .help = _("Do not filter escape codes")},
 	        {.sh = 'I', .type = HXTYPE_VAL, .ptr = &Opt.infod_start,
 		 .val = true, .help = _("Make statistics available through socket")},
 	        {.sh = 'i', .type = HXTYPE_VAL, .ptr = &Opt.infod_start,
@@ -791,7 +869,6 @@ static bool get_options(int *argc, const char ***argv)
 static void getopt_config(const struct HXoptcb *cbi)
 {
 	read_config(cbi->data);
-	return;
 }
 
 static void getopt_username(const struct HXoptcb *cbi)
@@ -800,7 +877,6 @@ static void getopt_username(const struct HXoptcb *cbi)
 		fprintf(stderr, _("No such user: %s\n"), cbi->data);
 		exit(EXIT_FAILURE);
 	}
-	return;
 }
 	
 static bool rate_limit(int counter, time_t delta)
